@@ -1,6 +1,6 @@
 use criterion::{Criterion, criterion_group, criterion_main};
-use std::hint::black_box;
 use rawler::ljpeg92::LjpegCompressor;
+use std::hint::black_box;
 
 fn generate_data(w: usize, h: usize, ncomp: usize) -> Vec<u16> {
   let mut img = vec![0; w * h * ncomp];
@@ -19,6 +19,13 @@ fn encode_ljpeg(img: &[u16], w: usize, h: usize, ncomp: usize) {
   let _result = state.encode().unwrap();
 }
 
+fn copy_data(img: &[u16], w: usize, h: usize) {
+  let mut result = vec![0; w * h];
+  for (dst, src) in result.chunks_exact_mut(w).zip(img.chunks_exact(w)) {
+    dst.copy_from_slice(src);
+  }
+}
+
 fn criterion_benchmark(c: &mut Criterion) {
   let mut group = c.benchmark_group("ljpeg-encoder");
   // Configure Criterion.rs to detect smaller differences and increase sample size to improve
@@ -31,6 +38,10 @@ fn criterion_benchmark(c: &mut Criterion) {
     b.iter(|| encode_ljpeg(black_box(data), black_box(3000), black_box(2000), black_box(1)))
   });
 
+  group.bench_with_input("copy_3000x2000", &x, |b, data| {
+    b.iter(|| copy_data(black_box(data), black_box(3000), black_box(2000)))
+  });
+
   group.finish();
 }
 
