@@ -276,7 +276,12 @@ impl<'a> Decoder for Cr3Decoder<'a> {
 
     // Load trak with raw MDAT section
     let moov_trak = self.moov_trak(raw_trak_id).ok_or(format!("Unable to get MOOV trak {}", raw_trak_id))?;
-    let (offset, size) = moov_trak.mdia.minf.stbl.get_sample_offset(sample_idx as u32 + 1).unwrap();
+    let (offset, size) = moov_trak
+      .mdia
+      .minf
+      .stbl
+      .get_sample_offset(sample_idx as u32 + 1)
+      .ok_or_else(|| RawlerError::DecoderFailed(format!("stbl sample not found")))?;
     debug!("RAW mdat offset: {}, len: {}", offset, size);
     // Raw data buffer
     let buf = file
@@ -286,8 +291,12 @@ impl<'a> Decoder for Cr3Decoder<'a> {
     let cmp1 = self.cmp1_box(raw_trak_id).ok_or(format!("CMP1 box not found for trak {}", raw_trak_id))?;
     debug!("cmp1 mdat hdr size: {}", cmp1.mdat_hdr_size);
 
-    let mut wb = cr3md.wb.unwrap_or([f32::NAN, f32::NAN, f32::NAN, f32::NAN]);
-    let whitelevel = cr3md.whitelevel.unwrap_or(u16::MAX);
+    let mut wb = cr3md.wb.unwrap_or_else(|| {
+      // This is known for R5 C CRM Standard-Raw files
+      log::warn!("No WB info in CR3 metadata found, fallback to 1.0 coefficients");
+      [1.0, 1.0, 1.0, f32::NAN]
+    });
+    let whitelevel = cr3md.whitelevel.unwrap_or(((1_u32 << self.camera.bps.unwrap_or(16)) - 1) as u16);
 
     // Special handling for CRM movie files
     if let Some(entry) = self.cmt3.get_entry(0x0001) {
