@@ -1,18 +1,6 @@
-use std::cmp;
-
-use image::EncodableLayout;
-use image::ImageBuffer;
-use image::Rgb;
-
 use crate::RawImage;
-use crate::alloc_image_ok;
-use crate::alloc_image_plain;
-use crate::bits::Endian;
-use crate::bits::LEu32;
-use crate::bits::LookupTable;
 use crate::cfa::*;
 use crate::decoders::*;
-use crate::decompressors::ljpeg::*;
 use crate::formats::tiff::Entry;
 use crate::formats::tiff::Rational;
 use crate::formats::tiff::Value;
@@ -21,7 +9,6 @@ use crate::imgop::Point;
 use crate::imgop::Rect;
 use crate::imgop::xyz::FlatColorMatrix;
 use crate::imgop::xyz::Illuminant;
-use crate::packed::*;
 use crate::tags::DngTag;
 use crate::tags::TiffCommonTag;
 
@@ -51,20 +38,6 @@ impl<'a> Decoder for DngDecoder<'a> {
     let height = fetch_tiff_tag!(raw, TiffCommonTag::ImageLength).force_usize(0);
     let cpp = fetch_tiff_tag!(raw, TiffCommonTag::SamplesPerPixel).force_usize(0);
     let bits = fetch_tiff_tag!(raw, TiffCommonTag::BitsPerSample).force_u32(0);
-
-    let mut image = match fetch_tiff_tag!(raw, TiffCommonTag::Compression).force_u32(0) {
-      1 => self.decode_uncompressed(file, raw, width * cpp, height, dummy)?,
-      7 => self.decode_compressed(file, raw, width * cpp, height, cpp, dummy)?,
-      c => return Err(RawlerError::DecoderFailed(format!("Don't know how to read DNGs with compression {}", c))),
-    };
-
-    if let Some(lintable) = raw.get_entry(TiffCommonTag::Linearization) {
-      if bits != 8 && fetch_tiff_tag!(raw, TiffCommonTag::Compression).force_u32(0) != 1 {
-        // 8 bit uncompressed data is already read delinearized
-        apply_linearization(&mut image, &lintable.value, bits);
-      }
-    }
-
     let orientation = Orientation::from_tiff(self.tiff.root_ifd());
 
     let mut cam = self.make_camera(raw, width, height)?;
@@ -84,9 +57,20 @@ impl<'a> Decoder for DngDecoder<'a> {
       _ => todo!(),
     };
 
-    let mut image = RawImage::new(cam, image, cpp, self.get_wb()?, photometric, blacklevel, whitelevel, dummy);
+    let raw_data = plain_image_from_ifd(raw, file)?;
+    let mut image = RawImage::new_with_data(
+      cam,
+      raw_data,
+      width * cpp,
+      height,
+      cpp,
+      self.get_wb()?,
+      photometric,
+      blacklevel,
+      whitelevel,
+      dummy,
+    );
     image.orientation = orientation;
-
     Ok(image)
   }
 
@@ -111,25 +95,10 @@ impl<'a> Decoder for DngDecoder<'a> {
 
   fn thumbnail_image(&self, file: &RawSource, _params: &RawDecodeParams) -> Result<Option<DynamicImage>> {
     if let Some(thumb_ifd) = Some(self.tiff.root_ifd()).filter(|ifd| ifd.get_entry(TiffCommonTag::NewSubFileType).map(|entry| entry.force_u16(0)) == Some(1)) {
-      let buf = thumb_ifd
-        .strip_data(&mut file.reader())
-        .map_err(|e| RawlerError::DecoderFailed(format!("Failed to get strip data: {}", e)))?
-        .into_iter()
-        .flatten()
-        .collect();
-      let compression = thumb_ifd.get_entry(TiffCommonTag::Compression).ok_or("Missing tag")?.force_usize(0);
-      let width = fetch_tiff_tag!(thumb_ifd, TiffCommonTag::ImageWidth).force_usize(0);
-      let height = fetch_tiff_tag!(thumb_ifd, TiffCommonTag::ImageLength).force_usize(0);
-      if compression == 1 {
-        return Ok(Some(DynamicImage::ImageRgb8(
-          ImageBuffer::<Rgb<u8>, Vec<u8>>::from_raw(width as u32, height as u32, buf).unwrap(), // TODO: remove unwraps
-        )));
-      } else {
-        let img = image::load_from_memory_with_format(&buf, image::ImageFormat::Jpeg).unwrap();
-        return Ok(Some(img));
-      }
+      Ok(Some(dynamic_image_from_ifd(thumb_ifd, file)?))
+    } else {
+      Ok(None)
     }
-    Ok(None)
   }
 
   fn full_image(&self, file: &RawSource, params: &RawDecodeParams) -> Result<Option<DynamicImage>> {
@@ -141,23 +110,7 @@ impl<'a> Decoder for DngDecoder<'a> {
         .iter()
         .find(|ifd| ifd.get_entry(TiffCommonTag::NewSubFileType).map(|entry| entry.force_u32(0)) == Some(1));
       if let Some(preview_ifd) = first_ifd {
-        let buf = preview_ifd
-          .strip_data(&mut file.reader())
-          .map_err(|e| RawlerError::DecoderFailed(format!("Failed to get strip data: {}", e)))?
-          .into_iter()
-          .flatten()
-          .collect();
-        let compression = preview_ifd.get_entry(TiffCommonTag::Compression).ok_or("Missing tag")?.force_usize(0);
-        let width = fetch_tiff_tag!(preview_ifd, TiffCommonTag::ImageWidth).force_usize(0);
-        let height = fetch_tiff_tag!(preview_ifd, TiffCommonTag::ImageLength).force_usize(0);
-        if compression == 1 {
-          return Ok(Some(DynamicImage::ImageRgb8(
-            ImageBuffer::<Rgb<u8>, Vec<u8>>::from_raw(width as u32, height as u32, buf).unwrap(), // TODO: remove unwraps
-          )));
-        } else {
-          let img = image::load_from_memory_with_format(&buf, image::ImageFormat::Jpeg).unwrap();
-          return Ok(Some(img));
-        }
+        return Ok(Some(dynamic_image_from_ifd(preview_ifd, file)?));
       }
     }
     Ok(None)
@@ -256,15 +209,22 @@ impl<'a> DngDecoder<'a> {
       .find_ifds_with_tag(TiffCommonTag::Compression)
       .into_iter()
       .filter(|ifd| {
-        let compression = (**ifd).get_entry(TiffCommonTag::Compression).unwrap().force_u32(0);
+        let compression = (**ifd)
+          .get_entry(TiffCommonTag::Compression)
+          .expect("This IFD must contains this tag")
+          .force_u32(0);
         let subsampled = match (**ifd).get_entry(TiffCommonTag::NewSubFileType) {
           Some(e) => e.force_u32(0) & 1 != 0,
           None => false,
         };
-        !subsampled && (compression == 7 || compression == 1 || compression == 0x884c)
+        !subsampled && (compression == 7 || compression == 8 || compression == 1 || compression == 0x884c || compression == 52546)
       })
       .collect::<Vec<&IFD>>();
-    Ok(ifds[0])
+    if let Some(first) = ifds.first() {
+      Ok(first)
+    } else {
+      Err(RawlerError::DecoderFailed(format!("TODO: Unsupported DNG compression")))
+    }
   }
 
   fn make_camera(&self, raw: &IFD, width: usize, height: usize) -> Result<Camera> {
@@ -379,15 +339,33 @@ impl<'a> DngDecoder<'a> {
 
   fn get_cfa(&self, raw: &IFD) -> Result<CFA> {
     let pattern = fetch_tiff_tag!(raw, TiffCommonTag::CFAPattern);
-    Ok(CFA::new_from_tag(pattern))
+    let cfa = CFA::new_from_tag(pattern);
+    // If DNG has active area, we need to calulate back the CFA pattern,
+    // because for DNG the CFAPattern is relative to ActiveArea and we
+    // use (0, 0) as starting point.
+    if let Some(active_area) = self.get_active_area_borders(raw) {
+      let top = active_area[0];
+      let left = active_area[1];
+      Ok(cfa.shift(left % cfa.width, top % cfa.height))
+    } else {
+      Ok(cfa)
+    }
+  }
+
+  fn get_active_area_borders(&self, raw: &IFD) -> Option<[usize; 4]> {
+    if let Some(crops) = raw.get_entry(DngTag::ActiveArea) {
+      let rect = [crops.force_usize(0), crops.force_usize(1), crops.force_usize(2), crops.force_usize(3)];
+      Some(rect)
+    } else {
+      // Ignore missing crops, at least some pentax DNGs don't have it
+      None
+    }
   }
 
   fn get_active_area(&self, raw: &IFD, width: usize, height: usize) -> Option<[usize; 4]> {
-    if let Some(crops) = raw.get_entry(DngTag::ActiveArea) {
-      let rect = [crops.force_usize(0), crops.force_usize(1), crops.force_usize(2), crops.force_usize(3)];
+    if let Some(rect) = self.get_active_area_borders(raw) {
       Some(Rect::new_with_dng(&rect).as_ltrb_offsets(width, height))
     } else {
-      // Ignore missing crops, at least some pentax DNGs don't have it
       None
     }
   }
@@ -440,140 +418,4 @@ impl<'a> DngDecoder<'a> {
 
     Ok(result)
   }
-
-  pub fn decode_uncompressed(&self, file: &RawSource, raw: &IFD, width: usize, height: usize, dummy: bool) -> Result<PixU16> {
-    let strips: Vec<&[u8]> = raw.strip_data_rawsource(file)?;
-    let strips_continous: Vec<u8>;
-    let src = if strips.len() == 1 {
-      strips[0]
-    } else {
-      strips_continous = strips.into_iter().flatten().copied().collect();
-      &strips_continous
-    };
-    match (raw.endian, fetch_tiff_tag!(raw, TiffCommonTag::BitsPerSample).force_u32(0)) {
-      // 16 bits, encoding depends on TIFF endianess
-      (Endian::Big, 16) => Ok(decode_16be(src, width, height, dummy)),
-      (Endian::Little, 16) => Ok(decode_16le(src, width, height, dummy)),
-      // 12 Bits, DNG spec says it must be always encoded as big-endian
-      (_, 12) => Ok(decode_12be(src, width, height, dummy)),
-      // 10 Bits, DNG spec says it must be always encoded as big-endian
-      (_, 10) => Ok(decode_10be(src, width, height, dummy)),
-      // 8 bits with linearization table
-      (_, 8) if raw.has_entry(TiffCommonTag::Linearization) => {
-        let linearization = fetch_tiff_tag!(self.tiff, TiffCommonTag::Linearization);
-        let curve = {
-          let mut points = vec![0_u16; 256];
-          for i in 0..256 {
-            points[i] = linearization.force_u32(i) as u16;
-          }
-          LookupTable::new(&points)
-        };
-        Ok(decode_8bit_wtable(src, &curve, width, height, dummy))
-      }
-      // 8 bits
-      (_, 8) => Ok(decode_8bit(src, width, height, dummy)),
-      // Generic MSB decoder for exotic packed bit sizes
-      (_, bps) if bps > 0 && bps < 16 => Ok(decode_generic_msb(src, width, height, bps, dummy)),
-      // Unhandled bits
-      (_, bps) => Err(format_args!("DNG: Don't know how to handle DNG with {} bps", bps).into()),
-    }
-  }
-
-  pub fn decode_compressed(&self, file: &RawSource, raw: &IFD, width: usize, height: usize, cpp: usize, dummy: bool) -> Result<PixU16> {
-    if let Some(offsets) = raw.get_entry(TiffCommonTag::StripOffsets) {
-      // We're in a normal offset situation
-      if offsets.count() != 1 {
-        return Err("DNG: files with more than one slice not supported yet".into());
-      }
-      let offset = offsets.force_u64(0);
-      let size = fetch_tiff_tag!(raw, TiffCommonTag::StripByteCounts).force_u64(0);
-      let src = file.subview(offset, size).unwrap();
-      let mut out = alloc_image_ok!(width, height, dummy);
-      let decompressor = LjpegDecompressor::new(src)?;
-      decompressor.decode(out.pixels_mut(), 0, width, width, height, dummy)?;
-      Ok(out)
-    } else if let Some(offsets) = raw.get_entry(TiffCommonTag::TileOffsets) {
-      // They've gone with tiling
-      let twidth = fetch_tiff_tag!(raw, TiffCommonTag::TileWidth).force_usize(0) * cpp;
-      let tlength = fetch_tiff_tag!(raw, TiffCommonTag::TileLength).force_usize(0);
-      let coltiles = (width - 1) / twidth + 1;
-      let rowtiles = (height - 1) / tlength + 1;
-      if coltiles * rowtiles != offsets.count() as usize {
-        return Err(format_args!("DNG: trying to decode {} tiles from {} offsets", coltiles * rowtiles, offsets.count()).into());
-      }
-      let buffer = file.as_vec().unwrap();
-      decode_threaded_multiline(
-        width,
-        height,
-        tlength,
-        dummy,
-        &(|strip: &mut [u16], row| {
-          let row = row / tlength;
-          for col in 0..coltiles {
-            // Calculate output width & length for current tile (right or bottom tile may have smaller dimension)
-            let owidth = cmp::min(width, (col + 1) * twidth) - col * twidth;
-            let olength = cmp::min(height, (row + 1) * tlength) - row * tlength;
-            let offset = offsets.force_usize(row * coltiles + col);
-            let src = &buffer[offset..];
-            let decompressor = LjpegDecompressor::new(src)?;
-
-            // If SOF width & height matches tile dimension, we can decode directly to strip
-            if decompressor.width() == twidth && decompressor.height() == tlength {
-              decompressor.decode(strip, col * twidth, width, owidth, olength, dummy)?;
-            }
-            // If SOF has other dimension but still same pixel count, we can decode
-            // by using a temporary tile buffer. This is the case if RGGB data is encoded
-            // by two input lines into one output line.
-            // Encoded data is aligned like this:
-            //   RGRGRGRGRGRGRGRGRGGBGBGBGBGBGBGBGBGB
-            //   RGRGRGRGRGRGRGRGRGGBGBGBGBGBGBGBGBGB
-            else if decompressor.width() * decompressor.height() == twidth * tlength {
-              // cps is already included in all values, so just compare them.
-              let mut tile = alloc_image_plain!(decompressor.width(), decompressor.height(), dummy);
-              // The stripwidth is of same width as the temporary tile.
-              let stripwidth = decompressor.width();
-              decompressor.decode(tile.pixels_mut(), 0, stripwidth, decompressor.width(), decompressor.height(), dummy)?;
-              // Copy lines from temporary tile to strip buffer, but limit it if destination strip
-              // is smaller than temporary tile (right/bottom tiles).
-              for (i, pix) in tile.pixels().chunks_exact(twidth).enumerate().take(olength) {
-                let start = (i * width) + (col * twidth);
-                strip[start..start + owidth].copy_from_slice(&pix[0..owidth]);
-              }
-            } else {
-              return Err(format!(
-                "ljpeg92 decoding failed: tile dimensions {}x{} not matching SOF dimensions {}x{}, dng cps: {}, sof cps: {}",
-                twidth,
-                tlength,
-                decompressor.width(),
-                decompressor.height(),
-                cpp,
-                decompressor.components(),
-              ));
-            }
-          }
-          Ok(())
-        }),
-      )
-      .map_err(RawlerError::DecoderFailed)
-    } else {
-      Err(RawlerError::DecoderFailed("DNG: didn't find tiles or strips".to_string()))
-    }
-  }
-}
-
-fn apply_linearization(image: &mut PixU16, tbl: &Value, bits: u32) {
-  match tbl {
-    Value::Short(points) => {
-      let table = LookupTable::new_with_bits(points, bits);
-      image.pixel_rows_mut().for_each(|row| {
-        let mut random = LEu32(row.as_bytes(), 0);
-        row.iter_mut().for_each(|p| {
-          *p = table.dither(*p, &mut random);
-        })
-      });
-    }
-    _ => {
-      panic!("Unsupported linear table");
-    }
-  }
 }
