@@ -978,7 +978,10 @@ impl<'a> IiqDecoder<'a> {
   fn get_lens_description(&self) -> Result<Option<&'static LensDescription>> {
     match self.lens_model()? {
       Some(model) => {
-        let resolver = LensResolver::new().with_lens_keyname(Some(model)).with_camera(&self.camera);
+        let resolver = LensResolver::new()
+          .with_mounts(&[String::from("phaseone")])
+          .with_lens_keyname(Some(model))
+          .with_camera(&self.camera);
         Ok(resolver.resolve())
       }
       None => Ok(None),
