@@ -2,10 +2,15 @@ use chrono::FixedOffset;
 use chrono::NaiveDateTime;
 use chrono::TimeZone;
 use image::DynamicImage;
+use image::ImageBuffer;
+use image::Luma;
+use image::Rgb;
 use log::debug;
 use log::warn;
+use multiversion::multiversion;
 use rayon::iter::IndexedParallelIterator;
 use rayon::iter::ParallelIterator;
+use rayon::prelude::*;
 use rayon::slice::ParallelSliceMut;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
@@ -17,19 +22,44 @@ use std::rc::Rc;
 use std::str::FromStr;
 use std::sync::Arc;
 use std::time::SystemTime;
-use toml::Value;
+use zerocopy::IntoBytes;
 
 use crate::Result;
+use crate::alloc_image_f32_plain;
 use crate::alloc_image_ok;
+use crate::alloc_image_plain;
 use crate::analyze::FormatDump;
+use crate::bits::LEu32;
+use crate::bits::LookupTable;
+use crate::bits::scale_u16;
+use crate::decompressors::Decompressor;
+use crate::decompressors::LineIteratorMut;
+use crate::decompressors::deflate::DeflateDecompressor;
+use crate::decompressors::jpeg::JpegDecompressor;
+use crate::decompressors::jpeg::LJpegDecompressor;
+use crate::decompressors::jpegxl::JpegXLDecompressor;
+use crate::decompressors::packed::PackedDecompressor;
 use crate::exif::Exif;
 use crate::formats::ciff;
 use crate::formats::jfif;
+use crate::formats::tiff::CompressionMethod;
+use crate::formats::tiff::ExtractFromIFD;
 use crate::formats::tiff::GenericTiffReader;
 use crate::formats::tiff::IFD;
+use crate::formats::tiff::PhotometricInterpretation;
+use crate::formats::tiff::SampleFormat;
+use crate::formats::tiff::Value;
+use crate::formats::tiff::ifd::DataMode;
 use crate::formats::tiff::reader::TiffReader;
+use crate::imgop::Dim2;
+use crate::imgop::Point;
+use crate::imgop::Rect;
 use crate::lens::LensDescription;
+use crate::pixarray::Pix2D;
+use crate::pixarray::PixF32;
 use crate::pixarray::PixU16;
+use crate::pixarray::SubPixel;
+use crate::pixarray::deinterleave2x2;
 use crate::rawsource::RawSource;
 use crate::tags::DngTag;
 
@@ -115,9 +145,9 @@ pub static CAMERAS_TOML: &str = include_str!(concat!(env!("OUT_DIR"), "/cameras.
 pub static SAMPLE: &str = "\nPlease submit samples at https://raw.pixls.us/";
 pub static BUG: &str = "\nPlease file a bug with a sample file at https://github.com/dnglab/dnglab/issues";
 
-const SUPPORTED_FILES_EXT: [&str; 28] = [
-  "ARI", "ARW", "CR2", "CR3", "CRM", "CRW", "DCR", "DCS", "DNG", "ERF", "IIQ", "KDC", "MEF", "MOS", "MRW", "NEF", "NRW", "ORF", "PEF", "RAF", "RAW", "RW2",
-  "RWL", "SRW", "3FR", "FFF", "X3F", "QTK",
+const SUPPORTED_FILES_EXT: [&str; 29] = [
+  "ARI", "ARW", "CR2", "CR3", "CRM", "CRW", "DCR", "DCS", "DNG", "ERF", "IIQ", "KDC", "MEF", "MOS", "MRW", "NEF", "NRW", "ORF", "ORI", "PEF", "RAF", "RAW",
+  "RW2", "RWL", "SRW", "3FR", "FFF", "X3F", "QTK",
 ];
 
 /// Get list of supported file extensions. All names
@@ -423,7 +453,7 @@ impl Orientation {
   }
 }
 
-pub fn ok_cfa_image(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], image: PixU16, dummy: bool) -> Result<RawImage> {
+pub(crate) fn ok_cfa_image(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], image: PixU16, dummy: bool) -> Result<RawImage> {
   assert_eq!(cpp, 1);
   Ok(RawImage::new(
     camera.clone(),
@@ -437,7 +467,7 @@ pub fn ok_cfa_image(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], image: PixU
   ))
 }
 
-pub fn ok_cfa_image_with_blacklevels(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], blacks: [u32; 4], image: PixU16, dummy: bool) -> Result<RawImage> {
+pub(crate) fn ok_cfa_image_with_blacklevels(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], blacks: [u32; 4], image: PixU16, dummy: bool) -> Result<RawImage> {
   assert_eq!(cpp, 1);
   let blacklevel = BlackLevel::new(&blacks, camera.cfa.width, camera.cfa.height, cpp);
   let img = RawImage::new(
@@ -453,7 +483,8 @@ pub fn ok_cfa_image_with_blacklevels(camera: Camera, cpp: usize, wb_coeffs: [f32
   Ok(img)
 }
 
-pub fn ok_cfa_image_with_black_white(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], black: u32, white: u32, image: PixU16, dummy: bool) -> Result<RawImage> {
+/*
+pub(crate) fn ok_cfa_image_with_black_white(camera: Camera, cpp: usize, wb_coeffs: [f32; 4], black: u32, white: u32, image: PixU16, dummy: bool) -> Result<RawImage> {
   assert_eq!(cpp, 1);
   let blacklevel = BlackLevel::new(&vec![black; cpp], 1, 1, cpp);
   let whitelevel = WhiteLevel::new(vec![white; cpp]);
@@ -469,6 +500,274 @@ pub fn ok_cfa_image_with_black_white(camera: Camera, cpp: usize, wb_coeffs: [f32
   );
   Ok(img)
 }
+   */
+
+pub(crate) fn dynamic_image_from_ifd(ifd: &IFD, rawsource: &RawSource) -> Result<DynamicImage> {
+  let tiff_width = fetch_tiff_tag!(ifd, TiffCommonTag::ImageWidth).force_usize(0);
+  let tiff_height = fetch_tiff_tag!(ifd, TiffCommonTag::ImageLength).force_usize(0);
+  let cpp = fetch_tiff_tag!(ifd, TiffCommonTag::SamplesPerPixel).force_usize(0);
+  let bits = fetch_tiff_tag!(ifd, TiffCommonTag::BitsPerSample).force_u32(0);
+  let image = plain_image_from_ifd(ifd, rawsource)?;
+  match image {
+    RawImageData::Integer(samples) => match bits {
+      8 => {
+        let buf: Vec<u8> = samples.into_iter().map(|p| p as u8).collect();
+        match cpp {
+          1 => Ok(DynamicImage::ImageLuma8(
+            ImageBuffer::<Luma<u8>, Vec<u8>>::from_raw(tiff_width as u32, tiff_height as u32, buf.to_vec())
+              .ok_or(RawlerError::DecoderFailed(format!("Create RGB thumbnail from strip failed")))?,
+          )),
+          3 => Ok(DynamicImage::ImageRgb8(
+            ImageBuffer::<Rgb<u8>, Vec<u8>>::from_raw(tiff_width as u32, tiff_height as u32, buf.to_vec())
+              .ok_or(RawlerError::DecoderFailed(format!("Create RGB thumbnail from strip failed")))?,
+          )),
+          _ => unimplemented!(),
+        }
+      }
+      9..=16 => {
+        let samples = if bits == 16 {
+          samples
+        } else {
+          samples.into_par_iter().map(|p| scale_u16(p, bits)).collect()
+        };
+        match cpp {
+          1 => Ok(DynamicImage::ImageLuma16(
+            ImageBuffer::<Luma<u16>, Vec<u16>>::from_raw((tiff_width) as u32, tiff_height as u32, samples)
+              .ok_or(RawlerError::DecoderFailed(format!("Create Y image failed")))?,
+          )),
+          3 => Ok(DynamicImage::ImageRgb16(
+            ImageBuffer::<Rgb<u16>, Vec<u16>>::from_raw((tiff_width) as u32, tiff_height as u32, samples)
+              .ok_or(RawlerError::DecoderFailed(format!("Create RGB image failed")))?,
+          )),
+          _ => unimplemented!(),
+        }
+      }
+      _ => unimplemented!(),
+    },
+    RawImageData::Float(samples) => match cpp {
+      3 => Ok(DynamicImage::ImageRgb32F(
+        // This may not work, rescaling required.
+        ImageBuffer::<Rgb<f32>, Vec<f32>>::from_raw(tiff_width as u32, tiff_height as u32, samples)
+          .ok_or(RawlerError::DecoderFailed(format!("Create RGB image failed")))?,
+      )),
+      _ => unimplemented!(),
+    },
+  }
+}
+
+pub(crate) fn plain_image_from_ifd(ifd: &IFD, rawsource: &RawSource) -> Result<RawImageData> {
+  let endian = ifd.endian;
+  let tiff_width = fetch_tiff_tag!(ifd, TiffCommonTag::ImageWidth).force_usize(0);
+  let tiff_height = fetch_tiff_tag!(ifd, TiffCommonTag::ImageLength).force_usize(0);
+  let cpp = fetch_tiff_tag!(ifd, TiffCommonTag::SamplesPerPixel).force_usize(0);
+  let bits = fetch_tiff_tag!(ifd, TiffCommonTag::BitsPerSample).force_u32(0);
+  let sample_format = SampleFormat::extract(ifd)?.unwrap_or(SampleFormat::Uint);
+  let compression = CompressionMethod::extract(ifd)?.unwrap_or(CompressionMethod::None);
+  let pi = PhotometricInterpretation::extract(ifd)?.unwrap_or(PhotometricInterpretation::BlackIsZero);
+
+  log::debug!(
+    "TIFF image: {}x{}, cpp={}, bits={}, compression={:?}, sample_format={:?}, data_mode={:?}, endian={:?}",
+    tiff_width,
+    tiff_height,
+    cpp,
+    bits,
+    compression,
+    sample_format,
+    ifd.data_mode()?,
+    endian,
+  );
+
+  let (decode_width, decode_height) = match ifd.data_mode()? {
+    DataMode::Strips => (tiff_width, tiff_height),
+    DataMode::Tiles => {
+      let twidth = fetch_tiff_tag!(ifd, TiffCommonTag::TileWidth).force_usize(0);
+      let tlength = fetch_tiff_tag!(ifd, TiffCommonTag::TileLength).force_usize(0);
+      (((tiff_width - 1) / twidth + 1) * twidth, ((tiff_height - 1) / tlength + 1) * tlength)
+    }
+  };
+
+  match sample_format {
+    SampleFormat::Uint => {
+      let mut pixbuf: PixU16 = alloc_image_plain!(decode_width * cpp, decode_height, false);
+      match (compression, ifd.data_mode()?) {
+        (CompressionMethod::None, DataMode::Strips) => {
+          decode_strips::<u16>(&mut pixbuf, rawsource, ifd, PackedDecompressor::new(bits, endian))?;
+        }
+        (CompressionMethod::None, DataMode::Tiles) => {
+          decode_tiles::<u16>(&mut pixbuf, rawsource, ifd, PackedDecompressor::new(bits, endian))?;
+        }
+        (CompressionMethod::ModernJPEG, DataMode::Strips) => {
+          if (pi == PhotometricInterpretation::YCbCr && bits == 8) || (pi == PhotometricInterpretation::BlackIsZero && bits == 8) {
+            decode_strips::<u16>(&mut pixbuf, rawsource, ifd, JpegDecompressor::new())?;
+          } else {
+            decode_strips::<u16>(&mut pixbuf, rawsource, ifd, LJpegDecompressor::new())?;
+          }
+        }
+        (CompressionMethod::ModernJPEG, DataMode::Tiles) => {
+          if (pi == PhotometricInterpretation::YCbCr && bits == 8) || (pi == PhotometricInterpretation::BlackIsZero && bits == 8) {
+            decode_tiles::<u16>(&mut pixbuf, rawsource, ifd, JpegDecompressor::new())?;
+          } else {
+            decode_tiles::<u16>(&mut pixbuf, rawsource, ifd, LJpegDecompressor::new())?;
+          }
+        }
+        (CompressionMethod::LossyJPEG, DataMode::Strips) => {
+          decode_strips::<u16>(&mut pixbuf, rawsource, ifd, JpegDecompressor::new())?;
+        }
+        (CompressionMethod::LossyJPEG, DataMode::Tiles) => {
+          decode_tiles::<u16>(&mut pixbuf, rawsource, ifd, JpegDecompressor::new())?;
+        }
+        (CompressionMethod::JPEGXL, DataMode::Strips) => decode_strips::<u16>(&mut pixbuf, rawsource, ifd, JpegXLDecompressor::new(bits))?,
+        (CompressionMethod::JPEGXL, DataMode::Tiles) => decode_tiles::<u16>(&mut pixbuf, rawsource, ifd, JpegXLDecompressor::new(bits))?,
+        _ => {
+          return Err(RawlerError::DecoderFailed(format!(
+            "Unsupported compression method: {:?}, storage: {:?}",
+            compression,
+            ifd.data_mode()?
+          )));
+        }
+      }
+
+      // We need to crop first, before we do stuff like deinterleaving.
+      // Padded pixels on output by LJPEG compression will corrupt deinterleave.
+      pixbuf = pixbuf.into_crop(Rect::new(Point::zero(), Dim2::new(tiff_width * cpp, tiff_height)));
+
+      if let Some(lintable) = ifd.get_entry(TiffCommonTag::Linearization) {
+        apply_linearization(&mut pixbuf, &lintable.value, bits);
+      }
+
+      // DNG 1.7.1 may store JPEG-XL data in interleaved format
+      let col_ilf = ifd.get_entry(DngTag::ColumnInterleaveFactor).map(|tag| tag.force_u16(0)).unwrap_or(1);
+      let row_ilf = ifd.get_entry(DngTag::RowInterleaveFactor).map(|tag| tag.force_u16(0)).unwrap_or(1);
+      match (row_ilf, col_ilf) {
+        (1, 1) => {}
+        (2, 2) => {
+          // Make sure pixbuf is properly cropped (e.g. LJPEG padding)
+          pixbuf = deinterleave2x2(&pixbuf)?;
+        }
+        _ => todo!(),
+      }
+      return Ok(RawImageData::Integer(pixbuf.into_inner()));
+    }
+    // Floating Point (IEEE) storage
+    SampleFormat::IEEEFP => {
+      let mut pixbuf: PixF32 = alloc_image_f32_plain!(decode_width * cpp, decode_height, false);
+      match (compression, ifd.data_mode()?) {
+        (CompressionMethod::None, DataMode::Strips) => {
+          decode_strips::<f32>(&mut pixbuf, rawsource, ifd, PackedDecompressor::new(bits, endian))?;
+        }
+        (CompressionMethod::None, DataMode::Tiles) => {
+          decode_tiles::<f32>(&mut pixbuf, rawsource, ifd, PackedDecompressor::new(bits, endian))?;
+        }
+        (CompressionMethod::Deflate, DataMode::Tiles) => {
+          let predictor = fetch_tiff_tag!(ifd, TiffCommonTag::Predictor).force_u16(0);
+          decode_tiles::<f32>(&mut pixbuf, rawsource, ifd, DeflateDecompressor::new(cpp, predictor, bits, endian))?;
+        }
+        _ => {
+          return Err(RawlerError::DecoderFailed(format!(
+            "Unsupported compression method: {:?}, storage: {:?}",
+            compression,
+            ifd.data_mode()?
+          )));
+        }
+      }
+
+      // We need to crop first, before we do stuff like deinterleaving.
+      // Padded pixels on output by LJPEG compression will corrupt deinterleave.
+      pixbuf = pixbuf.into_crop(Rect::new(Point::zero(), Dim2::new(tiff_width * cpp, tiff_height)));
+
+      // TODO: other corrections (see u16 code above)?
+
+      return Ok(RawImageData::Float(pixbuf.into_inner()));
+    }
+    _ => unimplemented!("other sample-formats"),
+  }
+}
+
+#[multiversion(targets("x86_64+avx+avx2", "x86+sse", "aarch64+neon"))]
+pub(super) fn decode_strips<'a, T>(pixbuf: &'a mut Pix2D<T>, file: &RawSource, raw: &IFD, dc: impl Decompressor<'a, T>) -> Result<()>
+where
+  T: SubPixel + 'a,
+{
+  let (strips, cont) = raw.strip_data(file)?;
+  let height = fetch_tiff_tag!(raw, TiffCommonTag::ImageLength).force_usize(0);
+  let rows_per_strip = raw.get_entry(TiffCommonTag::RowsPerStrip).map(|tag| tag.force_usize(0)).unwrap_or(height);
+  let line_width = pixbuf.width;
+  // If we have a continous buffer, we can optimize decompression of multiple strips.
+  if let Some(src) = cont {
+    // Some decompressors performing badly when skip_rows is > 0, because they need to
+    // decompress the skipped rows anyway.
+    if dc.strips_optimized() {
+      decode_threaded_prealloc(pixbuf, &|line, row| dc.decompress(src, row, std::iter::once(line), line_width))?;
+    } else {
+      dc.decompress(src, 0, pixbuf.pixel_rows_mut(), line_width)?;
+    }
+  } else {
+    decode_threaded_multiline_prealloc(pixbuf, rows_per_strip, &|lines, strip, _row| {
+      let src = strips[strip];
+      dc.decompress(src, 0, lines, line_width)
+    })?;
+  }
+
+  Ok(())
+}
+
+#[multiversion(targets("x86_64+avx+avx2", "x86+sse", "aarch64+neon"))]
+pub(super) fn decode_tiles<'a, T>(pixbuf: &'a mut Pix2D<T>, file: &RawSource, raw: &IFD, dc: impl Decompressor<'a, T>) -> Result<()>
+where
+  T: SubPixel + 'a,
+{
+  use crate::tiles::TilesMut;
+
+  let tiles_src = raw.tile_data(file)?;
+
+  let width = fetch_tiff_tag!(raw, TiffCommonTag::ImageWidth).force_usize(0);
+  let height = fetch_tiff_tag!(raw, TiffCommonTag::ImageLength).force_usize(0);
+  let cpp = fetch_tiff_tag!(raw, TiffCommonTag::SamplesPerPixel).force_usize(0);
+  let twidth = fetch_tiff_tag!(raw, TiffCommonTag::TileWidth).force_usize(0); // * cpp;
+  let tlength = fetch_tiff_tag!(raw, TiffCommonTag::TileLength).force_usize(0);
+  let coltiles = (width - 1) / twidth + 1;
+  let rowtiles = (height - 1) / tlength + 1;
+  if coltiles * rowtiles != tiles_src.len() as usize {
+    return Err(format_args!("DNG: trying to decode {} tiles from {} offsets", coltiles * rowtiles, tiles_src.len()).into());
+  }
+
+  let tiles = pixbuf
+    .data
+    .into_tiles_iter_mut(pixbuf.width / cpp, cpp, twidth, tlength)
+    .map_err(|_| RawlerError::DecoderFailed(format!("Unable to divide source image into tiles")))?;
+
+  let line_width = twidth * cpp;
+  tiles
+    .enumerate()
+    .par_bridge()
+    .map(|(tile_id, tile)| {
+      //eprintln!("Decode tile id {}", tile_id);
+      dc.decompress(tiles_src[tile_id], 0, tile.into_iter_mut(), line_width)
+    })
+    .collect::<std::result::Result<Vec<()>, _>>()?;
+
+  Ok(())
+}
+
+pub(crate) fn apply_linearization(image: &mut PixU16, tbl: &Value, bits: u32) {
+  match tbl {
+    Value::Short(points) => {
+      if points.is_empty() {
+        return;
+      }
+      let table = LookupTable::new_with_bits(points, bits);
+      image.par_pixel_rows_mut().for_each(|row| {
+        let mut random = LEu32(row.as_bytes(), 0);
+        row.iter_mut().for_each(|p| {
+          *p = table.dither(*p, &mut random);
+        })
+      });
+    }
+    _ => {
+      panic!("Unsupported linear table");
+    }
+  }
+}
 
 /// The struct that holds all the info about the cameras and is able to decode a file
 #[derive(Debug, Clone, Default)]
@@ -481,7 +780,7 @@ pub struct RawLoader {
 impl RawLoader {
   /// Creates a new raw loader using the camera information included in the library
   pub fn new() -> RawLoader {
-    let toml = match CAMERAS_TOML.parse::<Value>() {
+    let toml = match CAMERAS_TOML.parse::<toml::Value>() {
       Ok(val) => val,
       Err(e) => panic!("{}", format!("Error parsing cameras.toml: {:?}", e)),
     };
@@ -602,7 +901,7 @@ impl RawLoader {
 
     match GenericTiffReader::new(&mut rawfile.reader(), 0, 0, None, &[]) {
       Ok(tiff) => {
-        debug!("File is is TIFF file!");
+        debug!("File is a TIFF file!");
 
         macro_rules! use_decoder {
           ($dec:ty, $buf:ident, $tiff:ident, $rawdec:ident) => {
@@ -679,7 +978,7 @@ impl RawLoader {
     }
 
     // If all else fails see if we match by filesize to one of those CHDK style files
-    let data = rawfile.as_vec().unwrap();
+    let data = rawfile.as_vec()?;
     if let Some(cam) = self.naked.get(&data.len()) {
       return Ok(Box::new(nkd::NakedDecoder::new(cam.clone(), self)?));
     }
@@ -777,6 +1076,21 @@ where
   out
 }
 
+pub fn decode_threaded_prealloc<'a, T, F>(pixbuf: &'a mut Pix2D<T>, closure: &F) -> std::result::Result<(), String>
+where
+  F: Fn(&'a mut [T], usize) -> std::result::Result<(), String> + Sync,
+  T: SubPixel + 'a,
+{
+  let width = pixbuf.width;
+  pixbuf
+    .pixels_mut()
+    .par_chunks_exact_mut(width)
+    .enumerate()
+    .map(|(row, line)| closure(line, row))
+    .collect::<std::result::Result<Vec<()>, _>>()?;
+  Ok(())
+}
+
 pub fn decode_threaded_multiline<F>(width: usize, height: usize, lines: usize, dummy: bool, closure: &F) -> std::result::Result<PixU16, String>
 where
   F: Fn(&mut [u16], usize) -> std::result::Result<(), String> + Sync,
@@ -791,6 +1105,24 @@ where
   Ok(out)
 }
 
+/// Split pixbuf by `lines`.
+/// Caution: The last chunk may not be equal to `lines`.
+pub fn decode_threaded_multiline_prealloc<'a, T, F>(pixbuf: &'a mut Pix2D<T>, lines: usize, closure: &F) -> std::result::Result<(), String>
+where
+  F: Fn(&mut (dyn LineIteratorMut<'a, T>), usize, usize) -> std::result::Result<(), String> + Sync,
+  T: SubPixel + 'a,
+{
+  let width = pixbuf.width;
+  let chunksize = pixbuf.width * lines;
+  pixbuf
+    .pixels_mut()
+    .par_chunks_mut(chunksize)
+    .enumerate()
+    .map(|(strip, chunk)| closure(&mut chunk.chunks_exact_mut(width), strip, strip * lines))
+    .collect::<std::result::Result<Vec<()>, _>>()?;
+  Ok(())
+}
+
 /// This is used for streams where not chunked at line boundaries.
 pub fn decode_threaded_chunked<F>(width: usize, height: usize, chunksize: usize, dummy: bool, closure: &F) -> PixU16
 where
