@@ -454,11 +454,11 @@ where
     Ok(())
   }
 
-  pub fn subframe(&mut self, id: u32) -> SubFrameWriter<B> {
+  pub fn subframe(&mut self, id: u32) -> SubFrameWriter<'_, B> {
     SubFrameWriter::new(self, id, false)
   }
 
-  pub fn subframe_on_root(&mut self, id: u32) -> SubFrameWriter<B> {
+  pub fn subframe_on_root(&mut self, id: u32) -> SubFrameWriter<'_, B> {
     SubFrameWriter::new(self, id, true)
   }
 
@@ -669,7 +669,7 @@ where
         strip_sizes.push(std::mem::size_of_val(strip) as u32);
         strip_rows.push((strip.len() / (rawimage.width * rawimage.cpp)) as u32);
       }
-      subframe.ifd_mut().add_tag(TiffCommonTag::SampleFormat, &vec![3_u16; rawimage.cpp]); // Unsigned Integer// IEEE Float
+      subframe.ifd_mut().add_tag(TiffCommonTag::SampleFormat, &vec![3_u16; rawimage.cpp]); // IEEE Float
       subframe.ifd_mut().add_tag(TiffCommonTag::BitsPerSample, &vec![32_u16; rawimage.cpp]);
     }
   };
