@@ -9,6 +9,7 @@ use crate::Result;
 use crate::cfa::PlaneColor;
 use crate::imgop::raw::{correct_blacklevel, correct_blacklevel_cfa};
 use crate::imgop::{convert_from_f32_scaled_u16, convert_to_f32_unscaled};
+use crate::pixarray::SubPixel;
 use crate::{
   CFA,
   decoders::*,
@@ -40,7 +41,7 @@ impl WhiteLevel {
     if bits > 32 {
       panic!("Whitelevel can only be calculated for max. 32 bits, but {} bits given", bits);
     }
-    let level: u32 = (1 << bits) - 1;
+    let level: u32 = ((1_u64 << bits) - 1) as u32;
     Self(vec![level; cpp])
   }
 
@@ -265,7 +266,10 @@ impl RawImageData {
 }
 
 impl RawImage {
-  pub fn calc_black_levels(cfa: &CFA, blackareas: &[Rect], width: usize, _height: usize, image: &[u16]) -> Option<BlackLevel> {
+  pub fn calc_black_levels<T>(cfa: &CFA, blackareas: &[Rect], width: usize, _height: usize, image: &[T]) -> Option<BlackLevel>
+  where
+    T: SubPixel,
+  {
     let x = cfa.width * cfa.height;
     if x == 0 {
       return None;
@@ -286,13 +290,13 @@ impl RawImage {
           for col in area.p.x..area.p.x + area.d.w {
             //let color = cfa.color_at(row, col);
             let color = (row % cfa.height) * cfa.width + (col % cfa.width);
-            samples[color].avg += image[row * width + col] as f32;
+            samples[color].avg += image[row * width + col].as_f32();
             samples[color].count += 1;
           }
         }
       }
 
-      let blacklevels: Vec<u16> = samples.into_iter().map(|s| (s.avg / s.count as f32) as u16).collect();
+      let blacklevels: Vec<f32> = samples.into_iter().map(|s| (s.avg / s.count as f32)).collect();
 
       debug!("Calculated blacklevels: {:?}", blacklevels);
       // TODO: support other then RGGB levels
@@ -351,7 +355,7 @@ impl RawImage {
         if dummy {
           Some(BlackLevel::default())
         } else {
-          Self::calc_black_levels(&cam.cfa, &blackareas, image.width, image.height, image.pixels())
+          Self::calc_black_levels::<u16>(&cam.cfa, &blackareas, image.width, image.height, image.pixels())
         }
       })
       .unwrap_or_else(|| BlackLevel::zero(1, 1, cpp));
@@ -359,7 +363,7 @@ impl RawImage {
     let whitelevel = cam
       .make_whitelevel(cpp)
       .or(whitelevel)
-      .unwrap_or_else(|| panic!("Need whitelvel in config: {}", cam.clean_model));
+      .unwrap_or_else(|| panic!("Need whitelevel in config: {}", cam.clean_model));
 
     let crop_area = cam.crop_area.map(|area| Rect::new_with_borders(Dim2::new(pixel_width, image.height), &area));
 
@@ -388,6 +392,95 @@ impl RawImage {
     }
   }
 
+  #[doc(hidden)]
+  pub fn new_with_data(
+    cam: Camera,
+    image: RawImageData,
+    sample_width: usize,
+    height: usize,
+    cpp: usize,
+    wb_coeffs: [f32; 4],
+    photometric: RawPhotometricInterpretation,
+    blacklevel: Option<BlackLevel>,
+    whitelevel: Option<WhiteLevel>,
+    dummy: bool,
+  ) -> RawImage {
+    //assert_eq!(image.width % cpp, 0);
+    //assert_eq!(dummy, !image.is_initialized());
+    let pixel_width = sample_width / cpp;
+
+    let mut blackareas: Vec<Rect> = Vec::new();
+
+    let active_area = cam.active_area.map(|area| Rect::new_with_borders(Dim2::new(pixel_width, height), &area));
+
+    let blackarea_base = active_area.unwrap_or_else(|| Rect::new(Point::zero(), Dim2::new(sample_width, height)));
+
+    // For now, we only use masked areas when cpp is 1. For color images (RGB)
+    // like Canon SRAW, we ignore it (it isn't provided anyway).
+    if cpp == 1 {
+      // Build black areas
+      // First value (.0) is start and (.1) is length!
+      if let Some(ah) = cam.blackareah {
+        blackareas.push(Rect::new_with_points(
+          Point::new(blackarea_base.p.x, ah.0),
+          Point::new(blackarea_base.p.x + blackarea_base.d.w, ah.0 + ah.1),
+        ));
+      }
+      if let Some(av) = cam.blackareav {
+        blackareas.push(Rect::new_with_points(
+          Point::new(av.0, blackarea_base.p.y),
+          Point::new(av.0 + av.1, blackarea_base.p.y + blackarea_base.d.h),
+        ));
+      }
+    }
+
+    let blacklevel = cam
+      .make_blacklevel(cpp)
+      .or_else(|| if cam.find_hint("invalid_blacklevel") { None } else { blacklevel })
+      .or_else(|| {
+        if dummy {
+          Some(BlackLevel::default())
+        } else {
+          match &image {
+            RawImageData::Integer(pix) => Self::calc_black_levels(&cam.cfa, &blackareas, sample_width, height, pix),
+            RawImageData::Float(pix) => Self::calc_black_levels(&cam.cfa, &blackareas, sample_width, height, pix),
+          }
+        }
+      })
+      .unwrap_or_else(|| BlackLevel::zero(1, 1, cpp));
+
+    let whitelevel = cam
+      .make_whitelevel(cpp)
+      .or(whitelevel)
+      .unwrap_or_else(|| panic!("Need whitelvel in config: {}", cam.clean_model));
+
+    let crop_area = cam.crop_area.map(|area| Rect::new_with_borders(Dim2::new(pixel_width, height), &area));
+
+    RawImage {
+      camera: cam.clone(),
+      make: cam.make.clone(),
+      model: cam.model.clone(),
+      clean_make: cam.clean_make.clone(),
+      clean_model: cam.clean_model.clone(),
+      width: pixel_width,
+      height: height,
+      cpp,
+      bps: cam.real_bps,
+      wb_coeffs,
+      data: image,
+      blacklevel,
+      whitelevel,
+      xyz_to_cam: cam.xyz_to_cam,
+      photometric,
+      active_area,
+      crop_area,
+      blackareas,
+      orientation: Orientation::Normal, //cam.orientation, // TODO fixme
+      color_matrix: cam.color_matrix,
+      dng_tags: HashMap::new(),
+    }
+  }
+
   pub fn dim(&self) -> Dim2 {
     Dim2::new(self.width, self.height)
   }
