@@ -3,4 +3,8 @@ mod cameras;
 #[cfg(feature = "samplecheck")]
 mod common;
 #[cfg(feature = "samplecheck")]
+mod dng;
+#[cfg(feature = "samplecheck")]
 mod issues;
+#[cfg(feature = "samplecheck")]
+mod samples;
